#coding=utf-8
# 生成RSA私钥
def gen_prv(filepath, pwd = None):
    """
        生成pem格式私钥, 存放到filepath文件上
    """
def gen_pub(filepath, fp_prv, pwd = None):
    """
        从私钥文件fp_prv生成pem格式公钥, 保存到filepath文件上
    """
def load_prv(fp_prv, pwd = None):
    # 从磁盘上读取私钥
def load_pub(fp_pub):
    # 从磁盘上读取公钥
def gen_csr(filepath, fp_prv, conf = {}, pwd = None):
    '''
        生成csr文件，由fp_prv签名
    '''
def load_csr(filepath):
    # 读取csr
#要生成证书链，是从网站证书开始写入，往上写到根证书
"""
chains.cert文件结构:
网站证书
中间节点证书
...
根证书
"""
def sign_csr(filepath, fp_csr, fp_cert, fp_prv, conf = {}, pwd = None, add_chains = True):
    '''
        用cert证书签名csr，生成新的证书，add_chains如果为真，生成证书链
        证书链结构:
            网站证书
            中间节点证书
            ...
            根证书
    '''
def gen_cert(filepath, fp_prv, conf={}, pwd = None):
    """
    用私钥fp_prv签名公钥fp_pub和配置conf, 生成证书到filepath
    """
def loadf_certs(fp):
    # 从文件读取证书链
def load_certs(bs):
    # 读取证书链
def verify_certs_fp(fp, cas = None, verify_time=True):
    # 证书链校验，fp是需要校验的证书链文件，cas是根证书文件数组
def verify_certs(certs, cas = None, verify_time=True):
    # 证书链校验，certs是需要校验的证书链，cas是根证书数组

def verify_cert(cert, public_key=None, verify_time=True, hash_alg = None):
    # 证书校验
def load_cert(fp):
    # 读取证书


#coding=utf-8
from buildz import xf, argx, pyz, ioc, fz

from ..test import text_sfx, args, maps
class Deal:
    def deal(self):
        if len(args)<2:
            print("need params 1 to be dirpath")
            return
        dp = args[1]
        ct_fp = argx.get(maps, "f,file".split(","), None)
        ct = argx.get(maps, "c,content".split(","), None)
        depth = argx.get(maps, "d,depth".split(","), None)
        out_fp = argx.get(maps, "o,output".split(","), None)
        show = argx.get(maps, "s,show".split(","), False)
        prv = int(argx.get(maps, "p,prev".split(","), 10))
        aft = int(argx.get(maps, "a,aft".split(","), 10))
        shows = [prv, aft]
        if depth is not None:
            depth = int(depth)
        rst = fz.search(dp, ct_fp, ct, depth, show=show, shows=shows)
        l = len(rst)
        rs = "\n".join(rst)
        if out_fp is not None:
            with open(out_fp, 'w') as f:
                f.write(rs)
            print(f"result save in {out_fp}")
        else:
            print(rs)
        print(f"done search, find files: {l}")
    pass

pass



#

from .. import xf, fz
from .. import ioc
from ..base import Base
from ..ioc import wrap
from .. import logz
ns = wrap.ns("buildz.cache")
import os,re,json


@ns.obj(id="cache.save")
@ns.obj_args("ref, cache.file", "ref, log")
class Save(Base):
    def init(self, cache, log, fkey = "cache.save", fp = None):
        self.fkey = fkey
        self.cache = cache
        self.log = log.tag("Cache.Save")
        self.fp = fp
    def save(self):
        fp = self.fp
        if fp is None:
            self.log.warn(f"cache not save cause 'cache.save' is None")
            return False
        fz.makefdir(fp)
        rst  = self.cache.data
        rs = xf.dumps(rst, format=True).encode("utf-8")
        fz.write(rs, fp, 'wb')
        return True
    def call(self, maps, fp):
        fp = xf.get(maps, self.fkey, None)
        fp = self.cache.rfp(fp)
        self.fp = fp
        return self.save()

pass
class CacheKey(Base):
    def init(self, cache, key):
        self.cache = cache
        self.key = key
    def has(self):
        return self.cache.has(self.key)
    def set(self, val):
        return self.cache.set(self.key, val)
    def get(self):
        return self.cache.get(self.key)
    def call(self, *args):
        if len(args)==0:
            return self.get()
        elif len(args)==1:
            return self.set(args[0])
        else:
            assert 0
@ns.obj(id="cache.file")
@ns.obj_args("ref, fps, cache.js","ref, log, null", "ref, rfp.current.first, false", "ref, split, .", "ref, key.currents, buildz.cache.path.currents", "ref, key.basedir, buildz.cache.path.basedir", "ref, save.index, -1", "ref, save.auto, false")
class Cache(Base):
    def clean(self):
        self.data = {}
    def bind(self, key):
        return CacheKey(self, key)
    def has(self, key):
        ks = key.split(self.spt)
        return xf.has(self.data, ks)
    def get(self, key):
        ks = key.split(self.spt)
        return xf.gets(self.data, ks)
    def set(self, key, val):
        xf.sets(self.data, key.split(self.spt), val)
        if self.auto_save:
            self.save()
    def remove(self, key):
        xf.removes(self.data, key.split(self.spt))
    def init(self, fps="cache.js", log=None, current_first=False, spt = ".", key_currents = "buildz.cache.path.currents", key_basedir = "buildz.cache.path.basedir", save_index = -1, auto_save = False, load_replace=False, save_json = False):
        self.save_json = save_json
        if type(fps) not in (tuple, list):
            fps = [fps]
        self.fps = fps
        self.spt = spt
        self.current_first = current_first
        self.log = logz.make(log)(self)
        self.key_currents = key_currents
        self.key_basedir = key_basedir
        self.save_index = save_index
        self.load_replace = load_replace
        self.data = {}
        self.auto_save = auto_save
    def set_currents(self, dp):
        if type(dp)!=list:
            dp = [dp]
        self.set(self.key_currents, dp)
    def add_currents(self, dp):
        dps = self.get_currents()
        if dps is None:
            dps = []
        if dp in dps:
            return
        dps.append(dp)
        self.set_currents(dps)
    def get_currents(self):
        dps = self.get(self.key_currents)
        if dps is None:
            dps = []
        if type(dps)!=list:
            dps = [dps]
        return dps
    def set_basedir(self, dp):
        self.set(self.key_basedir, dp)
    def get_basedir(self):
        return self.get(self.key_basedir)
    def rfp(self, fp):
        if fz.is_abs(fp):
            return fp
        dps = [None,"."]
        cfps = self.get_currents()
        if cfps is not None:
            if self.current_first:
                dps = cfps+dps
            else:
                dps = dps+cfps
        basedir = self.get_basedir()
        if basedir is not None:
            dps = [basedir]+dps
        for dp in dps:
            _fp = fp
            if dp is not None:
                _fp = os.path.join(dp, fp)
            if os.path.isfile(_fp):
                return _fp
        if basedir is not None:
            fp = os.path.join(basedir, fp)
        return fp
    def save(self, fp = None, save_json=None):
        if fp is None:
            fp = self.fps[self.save_index]
        fp = self.rfp(fp)
        fz.makefdir(fp)
        rst = self.data
        if save_json is None:
            save_json = self.save_json
        if save_json:
            rs = json.dumps(rst, indent=4, ensure_ascii= False).encode("utf-8")
        else:
            rs = xf.dumps(rst, format=True).encode("utf-8")
        fz.write(rs, fp, 'wb')
    def load(self, fps = None):
        if fps is None:
            fps = self.fps
        data = {}
        for fp in fps:
            fp = self.rfp(fp)
            if os.path.isfile(fp):
                self.log.info(f"load cache from {fp}")
                xdata = xf.flush_maps(xf.loadf(fp),visit_list=True)
                xf.fill(xdata, data, replace=self.load_replace)
        xf.fill(data, self.data, replace=0)

pass

@ns.obj(id="cache.mem")
@ns.obj_args("ref, log, null")
class Mem(Cache):
    def init(self, log=None):
        super().init(None, log)

pass
@ns.obj(id="cache")
@ns.obj_args("ref, cache.mem", "ref, cache.file")
class Caches(Base):
    def init(self, mem, cache):
        self.cache = cache
        self.mem = mem
        self.caches = [mem, cache]
        self.set = mem.set
        self.remove = cache.remove
        self.call=cache.call
        self.rfp = cache.rfp
        self.get_currents = cache.get_currents
        self.add_currents = cache.add_currents
        self.set_currents = cache.set_currents
        self.set_basedir = cache.set_basedir
        self.get_basedir = cache.get_basedir
        self.save = cache.save
        self.load = cache.load
        self.has_file = cache.has
        self.has_mem = mem.has
    def get_file(self, key):
        return self.cache.get(key)
    def get_mem(self, key):
        return self.mem.get(key)
    def set_file(self, key, val):
        self.cache.set(key,val)
    def set_mem(self, key, val):
        self.mem.set(key, val)
    def remove_file(self, key):
        self.cache.remove(key)
    def remove_mem(self, key):
        self.mem.remove(key)
    def get(self, key):
        for cache in self.caches:
            if cache.has(key):
                return cache.get(key)
        return None
    def has(self, key):
        for cache in self.caches:
            if cache.has(key):
                return True
        return False

pass


from buildz import Base, xf, dz
from . import argz
from . import evalx
from ..iocz.ioc_deal.base import BaseEncape
class Fc(Base):
    def str(self):
        return str(self.fc)
    def init(self, fc=None):
        self.fc = fc
    def call(self, params):
        return self.fc(*params.args, **params.maps), 1
    @staticmethod
    def make(fc):
        if isinstance(fc, Fc):
            return fc
        return Fc(fc)
class RetFc(Fc):
    def call(self, params):
        return (params.args, params.maps), 1
def cal_eval(eval, inputs):
    if eval is None:
        return 1
    return eval(inputs)
def cal_args(args, inputs):
    if args is None:
        return inputs
    return args(inputs)
def deal_exp(fc, params, args):
    try:
        return fc(params)
    except argz.ArgExp as exp:
        if args is not None:
            exp = args.deal_exp(exp)
        raise exp
class Call(Fc):
    def str(self):
        return str(self.fcs)
    def init(self, fc, args=None, eval=None):
        if not dz.islist(fc):
            fc = [fc]
        fc = [self.make(k) for k in fc]
        self.fcs = fc
        self.args = args
        self.eval = eval
    def deal(self, params):
        rst = None,0
        for fc in self.fcs:
            val,mark_call = fc(params)
            if mark_call:
                rst = val, mark_call
        return rst
    def call(self, params):
        if not cal_eval(self.eval, params):
            return None, 0
        params = cal_args(self.args, params)
        return deal_exp(self.deal, params, self.args)
    def add(self, fc):
        self.fcs.append(self.make(fc))

pass


from .. import base
from .. import item
from .. import exp
from . import lr
import re
class ValDeal(base.BaseDeal):
    def prepare(self,mg):
        self.pt = mg.like(self.pt)
        self.type = mg.type
    def types(self):
        return ["str"]
    """
        正则表达式匹配
    """
    def init(self, pt, fc):
        st = "^"
        ed = "$"
        if pt[0]!=st:
            pt = st+pt
        if pt[-1]!=ed:
            pt = pt+ed
        self.pt = pt
        self.fc = fc
    def build(self, obj):
        val = obj.val
        if type(val) != self.type:
            return None
        if re.match(self.pt, val) is None:
            return None
        val = self.fc(val)
        return item.Item(val, obj.pos, type = "val", is_val = 1)

pass


from .. import Base, xf, dz
class ArgExp(Exception):
    def __repr__(self):
        return self.__str__()
    def __str__(self):
        return f"stype: {self.stype}, trs: {self.trs}, des: {self.des}"
    def __init__(self, stype, trs, des=None, prev = None):
        super().__init__()
        self.prev = prev
        self.stype = stype
        self.trs = trs
        self.des = des

pass
class ArgType:
    list = 'list'
    dict='dict'
    @staticmethod
    def stand(s):
        if ArgType.islist(s):
            return ArgType.list
        return ArgType.dict
    @staticmethod
    def islist(s):
        return s in ('list','l','lst','args')
    @staticmethod
    def isdict(s):
        return s in ('dict', 'd', 'dct', 'maps')

pass
class Params(Base):
    def str(self):
        return f"Params(args={self.args}, maps={self.maps})"
    def init(self, args=[], maps={}):
        self.args = args
        self.maps = maps
class Args(Base):
    '''
        参数映射基础类
    '''
    Type=ArgType
    def call(self, params):
        return params
    def deal(self, args, maps):
        params = self.call(Params(args,maps))
        return params.args, params.maps
    def deal_exp(self, exp):
        return exp

pass
class ArrArgs(Args):
    '''
        参数映射类的队列，顺序处理
    '''
    def str(self):
        return f"ArrArgs(args={self.args})"
    def init(self, args = None):
        if args is None:
            args = []
        self.args = args
    def add(self, item):
        self.args.append(item)
        return self
    def call(self, params):
        args, maps = params.args, params.maps
        i=0
        try:
            for i in range(len(self.args)):
                item = self.args[i]
                params = item(params)
        except ArgExp as exp:
            exp = self.deal_exp(exp, i-1)
            raise exp
        #params.args, params.maps = args, maps
        return params
    def deal_exp(self, exp, base=None):
        if base is None:
            base = len(self.args)-1
        for i in range(base, -1, -1):
            exp = self.args[i].deal_exp(exp)
        return exp
class RangeListArgs(Args):
    '''
        范围映射
    '''
    def str(self):
        return f"RangeArgs(base={self.base}, last={self.last}, min={self.min})"
    def init(self, base = 0, last = None, min=0):
        self.base = base
        self.last = last
        self.min = min
    def deal_exp(self, exp):
        trs = set()
        for key, vtype in exp.trs:
            if ArgType.islist(vtype):
                trs.add((key+self.base, vtype))
            else:
                trs.add((key, vtype))
        return ArgExp(exp.stype, trs, exp.des, exp)
    def call(self, params):
        args, maps = params.args, params.maps
        if self.last is None:
            args = args[self.base:]
        else:
            args = args[self.base:self.last]
        if len(args)<self.min:
            raise ArgExp("need", set([(self.base, 'list')]))
        return Params(args, maps)
        params.args, params.maps = args, maps
        return params


class ArgItem(Base):
    '''
        单个字段映射处理
    '''
    # vtype = 'dict' | 'list'
    def init(self, key, vtype=ArgType.dict, need = False, default = False, default_value = None, des = None,remove=True):
        self.vtype = vtype
        self.key = key
        self.need = need
        self.default = default
        self.default_value = default_value
        self.trs = []
        self.remove = remove
        if des is None:
            des = key
        self.des = des
    def add(self, key, vtype=ArgType.dict):
        self.trs.append((key, vtype))
        return self
    def deal_exp(self, exp):
        trs = set()
        for key, vtype in exp.trs:
            if key == self.key and vtype==self.vtype:
                trs.update(self.trs)
            else:
                trs.add((key, vtype))
        return ArgExp(exp.stype, trs, exp.des, exp)
    def fetch(self, key, vtype, set_args, args, maps):
        find = False
        val=None
        if ArgType.isdict(vtype):
            val,find = dz.dget(maps, key)
            if find:
                if self.remove:
                    dz.dremove(maps, key)
        else:
            if key in set_args:
                val = args[key]
                if self.remove:
                    set_args.remove(key)
                find = True
        return val, find
    def call(self, set_args, args, maps, rst_args, rst_maps):
        val = None
        find = False
        for key, vtype in self.trs:
            val, find = self.fetch(key, vtype, set_args, args, maps)
            if find:
                break
        if not find and self.default:
            find = True
            val = self.default_value
        if not find and self.need:
            raise ArgExp("need", set(self.trs), self.des)
        if not find:
            return
        if ArgType.isdict(self.vtype):
            xf.dset(rst_maps, self.key, val)
        else:
            rst_args[self.key] = val

pass
class ListFill:
    '''
        数组映射后，中间缺失位置如何处理
        exp=报错
        null=填充None
    '''
    exp='exp'
    null = 'null'
    default=exp
class TrsArgs(Args):
    def str(self):
        return f"TrsArgs(args={len(self.args)}, keep={self.keep}, fill = {self.list_fill})"
    '''
        数据映射
    '''
    def deal_exp(self, exp):
        for item in self.args:
            exp = item.deal_exp(exp)
        return exp
    def init(self, args=None, keep = False, list_fill = ListFill.default):
        super().init()
        if args is None:
            args = []
        self.args = args
        self.keep = keep
        self.list_fill = list_fill
    def add(self, item):
        self.args.append(item)
        return self
    def call(self, params):
        args, maps = params.args, params.maps
        rst_args, rst_maps = {}, {}
        set_args = set(range(len(args)))
        for item in self.args:
            item(set_args, args, maps, rst_args, rst_maps)
        if self.keep and len(set_args)>0:
            for i in set_args:
                if i not in rst_args:
                    rst_args[i] = args[i]
        l_args = 0
        if len(rst_args)>0:
            l_args= max(rst_args.keys())+1
        out_args = []
        for i in range(l_args):
            if i in rst_args:
                out_args.append(rst_args[i])
            else:
                if self.list_fill == ListFill.null:
                    out_args.append(None)
                else:
                    raise ArgExp("need", set([(i, ArgType.list)]))
        if self.keep:
            for k in maps:
                if k not in rst_maps:
                    rst_maps[k] = maps[k]
        return Params(out_args, rst_maps)
        params.args, params.maps = out_args, rst_maps
        return params

pass


from .. import base
from .. import item
from .. import exp
from . import spt
class SetDeal(spt.PrevSptDeal):
    """
        Map里的key-val读取
    """
    def init(self, spt):
        self.sp().init(spt, True)
    def deal(self, queue, stack):
        if len(stack)<3:
            return False
        if not stack[-1].check(is_val=1):
            return False
        if not stack[-3].check(is_val=1):
            return False
        if not stack[-2].check(type=self.id()):
            return False
        if stack[-2].check(is_keyval=1):
            return False
        val = stack.pop(-1)
        spt = stack.pop(-1)
        key = stack.pop(-1)
        if not key.check(is_val=1):
            print("key:", key)
            raise exp.FormatExp("not an usable key in key-val paris", key.pos, key.val)
        if not val.check(is_val=1):
            print("vak:", val)
            raise exp.FormatExp("not an usable val in key-val paris", val.pos, val.val)
        rst = [key.val, val.val]
        stack.append(item.DealItem(rst, key.pos, self.id(), is_keyval = 1))
        return True

pass


#
from .base import *
from .tdata import TagData
import os
class SysEnvs(Base):
    def init(self, ids):
        self.ids = ids
    def tget(self, key, ns=None, tag=None, id=None):
        rst = os.getenv(key)
        if rst is None and ns is not None:
            gkey = self.ids.id(self.ids(ns)+self.ids(key))
            rst = os.getenv(gkey)
        if rst is None:
            return None, TagData.Key.Pub, 0
        return rst, TagData.Key.Pub, 1




from buildz import xf,fz,pathz,logz,pyz
from buildz.netz import mhttp
import threading
confs = r"""
[
    {match=^/baidu, replace='www.baidu.com',ssl=true}
]
"""
rules = xf.loads(confs)
path = pathz.Path()
path.set("res", "./res")
log = logz.simple(path.res("gw/%Y%m%d.log"))
def gw():
    import sys,time
    ip = '127.0.0.1'
    port = 9999
    args = sys.argv[1:]
    if len(args)>0:
        ip = args.pop(0)
    if len(args)>0:
        port = int(args.pop(0))
    px = mhttp.Gateway((ip, port), rules, record=mhttp.MsgLog(log))
    th = threading.Thread(target=px,daemon=True)
    th.start()
    print(f"start on {(ip, port)}")
    while px.running:
        time.sleep(1)

#pyz.lc(locals(), caps)
pyz.lc(locals(), gw)

'''
cli:
import requests as rq
url="http://127.0.0.1:9999/baidu"
rp = rq.get(url)
print(rp)

'''




from .base import *
from ..ioc_deal import ref
class RefDeal(ref.RefDeal):
    def init(self, env=-1):
        super().init(env)
        conf = BaseConf()
        conf.ikey(1, 'ref', 'key,data'.split(","), need=1)
        conf.index(2, 'default')
        conf.ikey(3, 'env', 'profile'.split(','))
        self.update = conf

pass





from buildz import xf,fz,pathz,logz
from buildz.netz import mhttp
import threading
try:
    from buildz.netz import sslz
except:
    print(f"need package cryptography: pip install cryptography")
    raise

path = pathz.Path()
path.set("res", "./res")
fp_prv = path.res("ca.prv")
fp_cert = path.res("ca.crt")
ca_pwd = "test"


ca_conf = xf.loads(r"""
// 国家
country: CN
// 省份
provice: fujian
// 城镇
local: quanzhou
// 机构
org: buildz
// 域名
comman: buildz
// 邮箱
email: netz@buildz
// 是否是ca
ca: true
//dns: [localhost]
// 签名有效提前时间（签名多少天前就有效）
valid_before: 1
// 签名有效时间（签名后有效多少天）
valid: 3650
ca:true
""")
log = logz.simple(path.res("caps/%Y%m%d.log"))
fz.makedir(path.res())
def gen_cert():
    import os
    if os.path.isfile(fp_cert) and os.path.isfile(fp_prv):
        return
    sslz.gen_prv(fp_prv, ca_pwd)
    sslz.gen_cert(fp_cert,fp_prv, ca_conf, ca_pwd)


def caps():
    gen_cert()
    import sys,time
    ip = '127.0.0.1'
    port = 9999
    args = sys.argv[1:]
    if len(args)>0:
        ip = args.pop(0)
    if len(args)>0:
        port = int(args.pop(0))
    px = mhttp.CapsProxy((ip, port), fp_cert, fp_prv, ca_pwd, record=mhttp.MsgLog(log))
    th = threading.Thread(target=px,daemon=True)
    th.start()
    print(f"start on {(ip, port)}")
    while px.running:
        time.sleep(1)

#pyz.lc(locals(), caps)






#coding=utf-8
#from buildz.xf import code as codez
from buildz.base import Args
# class Args:
#     def size(self):
#         return len(self.args)+len(self.maps)
#     def str(self):
#         return f"<Args args={self.args}, maps={self.maps}>"
#     def __init__(self, args, maps):
#         self.args = args
#         self.maps = maps

# pass


cdef extern from "Python.h":
    object PyBytes_FromStringAndSize(const char* v, ssize_t len)
    object PyBytes_FromString(const char* v)

pass
cdef extern from "pc.h":
    ctypedef object (*fptr_create)(int type, void* dt, int val)
    ctypedef void (*fptr_list_add)(object map, object val)
    ctypedef void (*fptr_dict_set)(object map, object key, object val)
    ctypedef object (*fptr_exp)(const char* msg)
    object ploads_fcs(const char* s, fptr_create fc_create, fptr_dict_set fc_set, fptr_list_add fc_add, fptr_exp fc_error)
    object ploadx_fcs(const char* s, fptr_create fc_create, fptr_dict_set fc_set, fptr_list_add fc_add, fptr_exp fc_error)

# pass
cdef object fCreate(int type, void* dt, int ival)noexcept:
    #print("fCreate:")
    if type == 0:
        return None
    elif type == 1:
        return ival!=0
    # elif type == 101:
    #     return True
    # elif type == 102:
    #     return False
    elif type == 2:
        return int(PyBytes_FromString(<const char*>dt))
    elif type == 3:
        return float(PyBytes_FromString(<const char*>dt))
    elif type == 4:
        return PyBytes_FromString(<const char*>dt).decode("utf-8")
    #elif type == 401:
    #    return codez.ub2s(PyBytes_FromString(<const char*>dt))
    elif type == 5:
        return []
    elif type == 6:
        return {}
    elif type == 7:
        return Args(list(),dict())
    return None
pass
def fetchArgs(val):
    if type(val)==Args:
        if val.size()==len(val.lists):
            val = val.lists
        elif val.size()==len(val.dicts):
            val = val.dicts
    return val
cdef void fListAdd(object lst, object data)noexcept:
    #print("fListAdd:", lst, data)
    lst.append(data)
cdef void fListAddx(object lst, object data)noexcept:
    #print("fListAdd:", lst, data)
    if type(lst)==Args:
        lst = lst.args
    data = fetchArgs(data)
    lst.append(data)
cdef void fMapSet(object map, object key, object val)noexcept:
    #print("fMapSet:", map, key, val)
    if type(key)==list:
        key = tuple(key)
    map[key]=val
cdef void fMapSetx(object map, object key, object val)noexcept:
    #print("fMapSet:", map, key, val)
    if type(map)==Args:
        map = map.maps
    key = fetchArgs(key)
    val = fetchArgs(val)
    if type(key)==list:
        key = tuple(key)
    map[key]=val
cdef object fError(const char* msg)noexcept:
    #print("fError:")
    #print(f"error: msg={msg}")
    s = PyBytes_FromString(msg).decode("utf-8")
    exp_msg = f"error: msg={s}"
    return Exception(exp_msg)

pass
def loads(s,coding="utf-8"):
    if type(s)==str:
        s = s.encode(coding)
    rst = ploads_fcs(s, fCreate,fMapSet,fListAdd,fError)
    if type(rst)==Exception:
        raise rst
    if type(rst)==list and len(rst)==0:
        rst = ''
    if type(rst)==list and len(rst)==1:
        rst = rst[0]
    return rst

pass
def loadx(s,coding="utf-8", out_args=False):
    """
        和python版loadx区别：
            cpp版没有dict和list，全是Args
    """
    if type(s)==str:
        s = s.encode(coding)
    rst = ploadx_fcs(s, fCreate,fMapSetx,fListAddx,fError)
    if type(rst)==Exception:
        raise rst
    # if type(rst)==list and len(rst)==0:
    #     rst = ''
    if type(rst)==Args and rst.size()==0:
        rst = ''
    # if type(rst)==list and len(rst)==1:
    #     rst = rst[0]
    if type(rst)==Args and rst.size()==1 and len(rst.args)==1:
        rst = rst.args[0]
    if type(rst)==Args and not out_args:
        if rst.size()==len(rst.args):
            rst = rst.args
        elif rst.size()==len(rst.maps):
            rst = rst.maps
    return rst

pass




import sys
import os
import time
from buildz import xf
FP = "run.conf"
def find(arr, k, base = 0):
    for i in range(base, len(arr)):
        if arr[i]==k:
            return i
    return -1

pass
if __name__ == "__main__":
    from dv import build
    from dv.structz import CMD
else:
    from .dv import build
    from .dv.structz import CMD

pass

def loop_get(conf, key, loop, default=None, exp=False):
    key = xf.get(conf, key, default)
    if loop:
        while key in conf:
            key = conf[key]
    return key
def make(conf, loop=True):
    obj = conf
    dv = loop_get(obj, 'dv', loop)
    db_url = loop_get(obj, 'db', loop)
    assert dv is not None
    assert db_url is not None
    user = loop_get(obj, 'user', loop,None)
    pwd = loop_get(obj, 'pwd', loop,None)
    obj = build(dv, [db_url, user, pwd], obj)
    return obj
def cmd(conf, loop=True):
    return CMD(make(conf, loop))

pass
def test(fp):
    obj = xf.loads(xf.fread(fp))
    loop =  xf.g(obj, loop=True)
    dv = loop_get(obj, 'dv', loop)
    db_url = loop_get(obj, 'db', loop)
    assert dv is not None
    assert db_url is not None
    user = loop_get(obj, 'user', loop)
    pwd = loop_get(obj, 'pwd', loop)
    src = loop_get(obj, 'src', loop)
    out = loop_get(obj, 'out', loop)
    sqls = xf.fread(src)
    sqls = sqls.replace("\r\n", "\n").rstrip().split("\n")
    sqls_strip = [k.strip() for k in sqls]
    i = find(sqls_strip, "!!begin")
    bi = i
    if i < 0:
        bi=0
    j = find(sqls_strip, "!!end", bi)
    sqls = sqls[i+1:j]
    sqls = [k for k in sqls if k.lstrip()[:2]!="--"]
    sqls = "\n".join(sqls)
    sqls = sqls.split(";")
    print(f"[TESTZ] sqls:{sqls}")
    cmd = CMD(build(dv, [db_url, user, pwd], obj))
    cmd.dv.begin()
    print("[TESTZ] A")
    with open(out, 'wb') as f:
        f.write(b"[START SQL]\n\n")
    for sql in sqls:
        if sql.strip()=="":
            continue
        if sql.strip() == "exit":
            break
        print("[TESTZ] B")
        curr=time.time()
        rst = cmd.single(sql)
        cost = time.time()-curr
        print(f"[TESTZ] Cost: {cost} sec")
        rst += f"time cost: {cost} sec\n"
        with open(out, 'ab') as f:
            f.write(rst.encode("utf-8"))
    with open(out, 'ab') as f:
        f.write(b"\n[DONE SQL]")
    cmd.close()

pass
class Runner:
    def __init__(self, fp):
        self.fp = fp
    def init(self):
        self.obj = xf.loads(xf.fread(self.fp))
        self.fps = [self.fp, self.obj['src']]
        self.fps = [os.path.abspath(fp) for fp in self.fps]
        self.curr = [0,0]
    def run(self):
        while True:
            secs = [os.path.getmtime(fp) for fp in self.fps]
            if self.curr != secs:
                print(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()), "done update")
                self.init()
                self.curr = secs
                test(self.fp)
            time.sleep(float(self.obj['sec']))

pass

def run(fp):
    r = Runner(fp)
    r.init()
    r.run()

pass
def main():
    fp = FP
    if len(sys.argv)>1:
        fp = sys.argv[1]
    return run(fp)

pass

if __name__=="__main__":
    main()

pass



from buildz import argz,xf,Base,evalz,pathz
'''
python -m buildz.argz.test search /d/rootz
python -m buildz.argz.test_obj search /d/rootz f=asf c=test
'''

def search(dp, filepath = None, content = None):
    print(f"call in search: ({dp}, {filepath}, {content})")
    return f"call in search {dp}"

pass

args = argz.ArrArgs().add(
    argz.RangeListArgs(1)
)
trs = argz.TrsArgs().add(
    argz.ArgItem("filepath", "dict").add("f", "dict").add("fp", "dict").add("filepath", "dict")
).add(
    argz.ArgItem(0, "list",need=True,des="path").add(0, "list")
).add(
    argz.ArgItem("content", "dict").add("c", "dict").add("ct", "dict").add("content", "dict")
)
args.add(trs)
fc = argz.ArgsCall(search, args)
judge = lambda obj:obj[0][0]=='search'
fc = argz.EvalCall(fc, judge)
fcs = argz.Calls([fc])



main_args = argz.ArrArgs().add(argz.RangeListArgs(1))
main = argz.ArgsCall(fcs, main_args)


ins = xf.args(base=0)
args, maps = ins.args, ins.maps
rst = main(args, maps)
print(rst)


from .. import base
from .. import item
from .. import exp
from . import lr
from . import reval
class Fcs:
    def __init__(self):
        self.maps = {}
    def set(self, _type, fc):
        self.maps[_type] = fc
    def __call__(self, val, _type):
        if _type not in self.maps:
            raise Exception("unreginize type:"+_type)
        return self.maps[_type](val)

pass


class LRValDeal(lr.LRDeal):
    """
    """
    def init(self, left, right, fc):
        super().init(left, right, 'lrval', False)
        self.fc = fc
    def build_arr(self, arr, arr_pos):
        rst = []
        #pos = self.arr_pos(arr)#(arr[0].pos[0], arr[-1].pos[-1])
        if len(arr)!=3:
            raise exp.Exp(f"error in lrval:{arr}", arr_pos)
        for _item in arr[::2]:
            if type(_item.val)!=str:
                raise exp.Exp(f"error in list:{_item}", _item.pos)
            rst.append(_item.val)
        try:
            val = self.fc(rst[0], rst[1])
        except Exception as exp1:
            print("exp:", exp1)
            raise exp.Exp(f"error in lrval fc: {self.fc}({rst}): {exp1} ", pos)
        return item.Item(val, arr_pos, type='val', is_val = 1)
    def build_arrx(self, arr, arr_pos):
        return self.build(obj)

pass



#coding=utf-8
from . import pyz
def fcBase(cls, lst=0, keys = tuple()):
    class _Base(cls):
        def str(self):
            return str(self.__class__)
        def repr(self):
            return self.str()
        def __str__(self):
            return self.str()
        def __repr__(self):
            return self.repr()
        def __call__(self, *a, **b):
            return self.call(*a, **b)
        def call(self, *a, **b):
            return None
        def init(self, *a, **b):
            pass
        def __init__(self, *a, **b):
            sa = a[:lst]
            a = a[lst:]
            sb = {}
            for k in keys:
                if k in b:
                    sb[k] = b[k]
                    del b[k]
            super().__init__(*sa, **sb)
            self.init(*a, **b)
    return _Base

class Base:
    def str(self):
        return str(self.__class__)
    def __str__(self):
        return self.str()
    def __repr__(self):
        return self.__str__()
    def __init__(self, *args, **maps):
        self.init(*args, **maps)
    def __call__(self, *args, **maps):
        return self.call(*args, **maps)
    def init(self, *args, **maps):
        pass
    def call(self, *args, **maps):
        return self.deal(*args, **maps)
    def deal(self, *args, **maps):
        return None

pass

class WBase(Base):
    def _open(self):
        pass
    def open(self):
        self._open()
        return pyz.with_out(self.close, True)
    def _close(self):
        pass
    def close(self, exc_type, exc_val, exc_tb):
        self._close()

pass

class With(Base):
    def init(self, args=False):
        self.args = args
    def call(self, cls):
        def _open(obj):
            pass
        def open(obj):
            obj._open()
            return pyz.with_out(obj.close, self.args)
        def _close(obj):
            pass
        def close(obj):
            obj._close()
        cls._open = _open
        cls._close = _close
        cls.open = open
        cls.close = close
        return cls

pass

class Args(Base):
    def size(self):
        return len(self.args)+len(self.maps)
    def str(self):
        return f"<Args args={self.args}, maps={self.maps}>"
    @property
    def lists(self):
        return self.args
    @lists.setter
    def lists(self, val):
        self.args=val
    @lists.deleter
    def lists(self):
        del self.args
    @property
    def dicts(self):
        return self.maps
    @dicts.setter
    def dicts(self, val):
        self.maps=val
    @dicts.deleter
    def dicts(self):
        del self.maps
    def init(self, args, maps):
        self.args = args
        self.maps = maps

pass


#coding=utf-8
import os
class FileInfo:
    def __repr__(self):
        return self.__str__()
    def __str__(self):
        return f"<path='{self.path}', rpath='{self.rpath}' isdir={self.isdir}>"
    def __init__(self, path, rpath, base_dir, isdir, empty_dir=False, visit_dir = False):
        self.path = path
        self.rpath = rpath
        self.base_dir = base_dir
        self.isdir = isdir
        self.empty_dir = empty_dir
        self.visit_dir = visit_dir
        self.name = os.path.basename(path)
        self.dirpath = os.path.dirname(path)
        self.rdirpath = os.path.dirname(rpath)

pass
#列出文件夹下所有文件和文件更新时间
#文件访问处理方法
#建议visit读取文件
#catch处理目录访问异常
class FileDeal:
    def __init__(self, *argv, **maps):
        self.init(*argv, **maps)
    def result(self):
        return None
    def reset(self, filepath, depth=0):
        pass
    def init(self, *argv, **maps):
        pass
    def work(self, *argv, **maps):
        return self.dirs(*argv, **maps)
    def dirs(self, filepath, depth = 0):
        self.reset(filepath, depth)
        dirs(filepath, self, depth)
        return self.result()
    def visit(self, fileinfo, depth):
        return True
    def catch(self, filepinfo, depth, exp):
        pass
    def deal(self, filepinfo, depth, exp = None):
        if exp is None:
            return self.visit(filepinfo, depth)
        else:
            return self.catch(filepinfo, depth, exp)
    def __call__(self, *argv, **maps):
        return self.deal(*argv, **maps)

pass
#遍历文件／文件夹filepath
def dirs(filepath, fc=FileDeal(), depth = 0, base_dir = None, rpath = None):
    if base_dir is None:
        base_dir = filepath
    if rpath is None:
        rpath = ""
    isdir = os.path.isdir(filepath)
    finfo = FileInfo(filepath, rpath, base_dir, isdir)
    visit = fc(finfo, depth)
    if isdir and visit:
        try:
            files = os.listdir(filepath)
        except Exception as exp:
            finfo = FileInfo(filepath, rpath, base_dir, isdir, False, True)
            fc(finfo, depth, exp)
            return
        if len(files)==0:
            finfo = FileInfo(filepath, rpath, base_dir, isdir, True, True)
            fc(finfo, depth)
        files = [
            [
                os.path.join(filepath, file),
                os.path.join(rpath, file)
            ]
            for file in files]
        [dirs(fp[0], fc, depth+1, base_dir, fp[1]) for fp in files]

pass





from . import proxy,mhttp, mhttps
import socket, threading
log = mhttp.log
from buildz import xf
import ssl,re
from os.path import join, isfile
import traceback
from ...iocz.conf import conf as confz
from ... import dz,pyz
class GWDealer(proxy.ProxyDealer):
    conf = confz.Conf()
    conf.ikey(0, 'match', need=1)
    conf.ikey(1, 'replace', need=1)
    conf.ikey(2, 'ssl')
    conf.ikey(3, 'type', 'match_type,http_type'.split(","))
    """
    rules:[
        [match, replace, [opt|ssl], [opt|match_type]]
        {match=match, replace=relace, ssl=false, match_type='GET'}
    ]
    """
    def init(self, skt, rules, verify_context, context, channel_read_size=1024000, record=None):
        self.context = context
        if context is not None:
            skt = context.wrap_socket(skt, server_side=True)
        rules = [self.conf(k)[0] for k in rules]
        self.rules = rules
        self.verify_context = verify_context
        super().init(skt, channel_read_size, record=record, default_deal = self.maps_deal)
    def match(self, line, headers):
        http_type, url, protocol = line
        for item in self.rules:
            pt, rep, mark_ssl, match_type = dz.g(item, match=None, replace=None, ssl=False, type=http_type)
            if match_type != http_type:
                continue
            rst = re.findall(pt, url)
            if len(rst)==0:
                continue
            match = rst[0]
            i = url.find(match)
            pfx, sfx = url[:i], url[i+len(match):]
            rurl = pfx+rep+sfx
            log.debug(f"url match: {rst} for {url}, replace to {rurl} with i={i}")
            return rurl, mark_ssl
        return None, False
    def ssl_connect(self, addr):
        return mhttps.wrap(self.verify_context, mhttp.WSocket.Connect(addr), False)
    def maps_deal(self, skt_cli, line, headers, data_size, skt=None):
        http_type, url, protocol = line
        rel_url, mark_ssl = self.match(line, headers)
        if rel_url is None:
            if data_size>0:
                data = skt_cli.read(data_size)
            if mhttp.check_chunked(headers):
                self.simple_chunked(skt_cli)
            return self.error(skt_cli)
        if mark_ssl:
            prev = "https"
        else:
            prev = "http"
        host = rel_url.split("?")[0].split("/")[0]
        rel_url = prev+"://"+rel_url
        for key in "Host,Origin".split(","):
            if key in headers:
                headers[key] = host
        if 'Referer' in headers:
            ref = headers['Referer']
            prev, src_host, path = mhttp.spt_url(ref)
            ref = mhttp.comb_url(prev, host, path)
            headers["Referer"] = ref
            log.info(f"[REF] host: {host}, ref: {ref}")
        line = http_type, rel_url, protocol
        fc_connect = None
        if mark_ssl:
            fc_connect = self.ssl_connect
        return self.default_deal(skt_cli, line, headers, data_size, skt, fc_connect)


class Gateway(proxy.Proxy):
    '''
        类似nginx的反向代理
    '''
    def init(self, addr, rules, fp_cert=None, fp_prv=None, password = None,listen=5, record=None, cafile=None, capath=None, cadata=None,check_hostname=True):
        super().init(addr, listen, record)
        self.rules = rules
        self.verify_context = mhttps.load_verify_context(cafile, capath, cadata,check_hostname)
        self.context = None if fp_cert is None else mhttps.load_server_context(fp_cert, fp_prv, password)
    def make_dealer(self, skt, addr):
        return GWDealer(skt, self.rules, self.verify_context, self.context, record=self.record.clone())

def test():
    import sys,time
    from buildz import xf
    conf_fp = sys.argv[1]
    conf = xf.loadf(conf_fp)
    addr = tuple(conf['addr'])
    rules = conf['rules']
    px = Gateway(addr, rules)
    th = threading.Thread(target=px,daemon=True)
    th.start()
    print(f"start on {(ip, port)}")
    while px.running:
        time.sleep(1)
pyz.lc(locals(), test)



#
from ..ioc.mg import Manager, GetKey, Ids
from ... import dz,xf
from ..conf_deal.deal import DealDeal
from .unit import ConfUnit
spt = Ids(".")
class DLKey(GetKey):
    @staticmethod
    def Load(conf):
        keys, kfind = dz.dget(conf, spt("dict.key"))
        lids, lfind = dz.dget(conf, spt("list.index"))
        default, find = dz.dget(conf, 'default', None)
        return DLKey(keys, lids, default)
    def init(self, keys, indexes, default=None):
        super().init()
        self.keys = dz.tolist(keys)
        self.indexes = dz.tolist(indexes)
        self.default = default
    def list_get(self, conf):
        for indexes in self.indexes:
            obj, find = dz.dget(conf, indexes)
            if find:
                return obj, find
        return self.default, 0
    def dict_get(self, conf):
        for keys in self.keys:
            obj, find = dz.dget(conf, keys)
            if find:
                return obj, find
        return self.default, 0
    def call(self, conf):
        if dz.islist(conf):
            return self.list_get(conf)
        return self.dict_get(conf)
    def list_fill(self, conf, key):
        for indexes in self.indexes:
            obj, find = dz.dget(conf, indexes)
            if find:
                dz.dset(conf, indexes, key)
                return
        assert 0
    def dict_fill(self, conf, key):
        for keys in self.keys:
            obj, find = dz.dget(conf, keys)
            if find:
                dz.dset(conf, keys, key)
                return
        dz.dset(conf, self.keys[0], key)
    def fill(self, conf, key):
        if dz.islist(conf):
            self.list_fill(conf, key)
        else:
            self.dict_fill(conf, key)
default_conf = None
s_default_conf = r"""
    {
        id.spt: '.'
        env: {
            spt: '.'
            orders: [local, args, conf, sys]
        }
        conf:{
            dict.key = 'id'
            list.index = [(0,1)]
            default: null
        }
        deal: {
            spt: '.'
            dict.key = type
            list.index = ((0,0),0)
            default: null
        }
        tag: {
            dict.key='tag'
        }
    }
"""
def init():
    global default_conf
    if default_conf is not None:
        return
    default_conf = xf.loads(s_default_conf)
    default_conf = dz.flush_maps(default_conf)
class ConfManager(Manager):
    def init(self, conf=None):
        init()
        spt = Ids(".")
        if conf is None:
            conf = {}
        if type(conf)==str:
            conf = xf.loads(conf)
        conf = dz.flush_maps(conf)
        dz.fill(default_conf, conf, 0)
        id_spt, find = dz.dget(conf, spt("id.spt"),".")
        ids = Ids(id_spt)
        deal_spt, find = dz.dget(conf, spt("id.spt"), id_spt)
        deal_ids = Ids(deal_spt)
        env_ids, find = dz.dget(conf, spt("env.spt"), id_spt)
        deal_key = DLKey.Load(conf['deal'])
        conf_key = DLKey.Load(conf['conf'])
        tag_key = DLKey.Load(conf['tag'])
        env_orders, find = dz.dget(conf, spt("env.orders"), None)
        super().init(ids, deal_key, conf_key, tag_key, deal_ids, env_ids, env_orders)
        self.set_deal('deal', DealDeal())
    def add_conf(self, conf):
        unit = ConfUnit(conf, self)
        return unit

pass



from buildz import xf, Base
class Translate(Base):
    """
    数据映射
    conf:
    main: [
        null, mdict
        [

        ]
    ]
    main: [
        [data.total, total] // [data.total, total, key]
        [data.diff, datas, list, translates]
    ]
    or: 
    main: {
        data.total: total
        data.diff: [datas, list, translates]
    }
    translates: {
        ...
    }
    """
    def init(self, conf, src_spt=".",dst_spt = ".", inline = False, remove_src = False, vars = None):
        self.conf = conf
        self.src_spt = src_spt
        self.dst_spt = dst_spt
        self.inline = inline
        self.remove_src = remove_src
        self.cache_conf = {}
        self.vars = vars
    def kfc(self, fckey, val, vars=None):
        fc = self.var(fckey, vars)
        #print(f"[TESTZ] kfc: fckey: {fckey}, val: {val}, fc: {fc}")
        return fc(val)
    def var(self, key, vars=None):
        #print(f"[TESTZ] vars: {vars}")
        if vars is None:
            vars = self.vars
        #print(f"[TESTZ] vars: {vars}")
        if vars is None:
            raise Exception(f"var not found: {key}")
        if type(vars)==dict:
            if key not in vars:
                raise Exception(f"var not found: {key}")
            return vars[key]
        if callable(vars):
            return vars(key)
        return vars.get(key)
    def call(self, data, key = "main", vars=None):
        trs = xf.get(self.conf, key)
        #print(f"[TESTZ] trs: {trs}")
        trs = expand(trs)
        #print(f"[TESTZ] expand.trs: {trs}")
        return self.deal(data, trs,vars)
    def conf_get(self, key):
        if key in self.cache_conf:
            return self.cache_conf[key]
        assert key in self.conf
        rst = self.conf[key]
        rst = dict2list(rst)
        self.cache_conf[key] = rst
        return rst
    def src_get(self, data, key):
        if self.src_spt is not None:
            key = key.split(self.src_spt)
        return xf.gets(data, key)
    def dst_set(self, data, key, val):
        #print(f"[TESTZ] key: {key}")
        if self.dst_spt is not None:
            key = key.split(self.dst_spt)
        return xf.sets(data, key, val)
    def src_has(self, data, key):
        if self.src_spt is not None:
            key = key.split(self.src_spt)
        return xf.has(data, key)
    def src_remove(self, data, key):
        if self.src_spt is not None:
            key = key.split(self.src_spt)
        return xf.removes(data, key)
    def get_type(self, trs):
        if len(trs)==2:
            if type(trs[1]) not in (list, dict):
                return "key"
            return trs[0]
        return trs[2]
    def deal(self, data, trs, vars=None):
        """
            type: key, 
            conf = [
                key, target_key, type, info
            ]
            key: return [target_key, target_val, exists]
        """
        # trs = xf.get(conf, key)
        # trs = dict2list(trs)
        _type = self.get_type(trs)
        #print(f"[TESTZ] deal.trs: {trs}")
        if _type in ("list", "dict"):
            info = trs[-1]
            if type(info) == str:
                info = self.conf_get(info)
            trs[-1] = info
        if _type == "list":
            _trs = trs[-1]
            _trs = expand(_trs)
            if self.inline:
                rst = data
            else:
                rst = [None]*len(data)
            for i in range(len(data)):
                dt = data[i]
                _rst = self.deal(dt, _trs, vars)
                rst[i] = _rst
        elif _type == "dict":
            if self.inline:
                rst = data
            else:
                rst = {}
            for _trs in trs[-1]:
                #print(f"[TESTZ] _trs: {_trs}")
                _trs = expand(_trs)
                #print(f"[TESTZ] expand_trs: {_trs}")
                src_key, target_key = _trs[:2]
                _ctype = self.get_type(_trs)
                if _ctype == "val":
                    _rst = _trs[3]
                elif _ctype == "var":
                    _rst = self.var(_trs[3], vars)
                else:
                    if not self.src_has(data, src_key):
                        continue
                    val = self.src_get(data, src_key)
                    if _ctype == "key":
                        _rst = val
                    elif _ctype == "kfc":
                        _rst = self.kfc(_trs[3], val, vars)
                    else:
                        _rst = self.deal(val, _trs, vars)
                if self.inline and self.remove_src and _ctype not in ("val", "var"):
                    self.src_remove(data, src_key)
                    #del data[src_key]
                self.dst_set(rst, target_key, _rst)
                #rst[target_key] = _rst
        else:
            raise Exception("error")
        return rst
def dict2list(trs):
    if type(trs)==dict:
        tmp = []
        for k,v in trs.items():
            if type(v) not in [list, tuple]:
                v = [v]
            if type(v)==tuple:
                v = list(v)
            if len(v)==1:
                v = v+["key"]
            v = [k]+v
            tmp.append(v)
        trs = tmp
    return trs
def expand(trs):
    trs = dict2list(trs)
    if len(trs)>0:
        k = trs[0]
        if type(k) in (list, tuple):
            trs = ['dict', trs]
    if len(trs)>1:
        val = trs[-1]
        if type(val)==dict:
            trs[-1] = dict2list(val)
    return trs

pass


#

from buildz.xz import trs
from buildz import xf

data = xf.loadf("data.js")
conf = xf.loadf("conf.js")

obj = trs.Translate(conf, inline = 1, remove_src = 1, vars = {"x":"test"})
rst = obj(data)
print(rst)





from .base import *
from ..ioc.unit import Unit
class WrapFcs:
    def __init__(self, unit, fcs=None):
        if fcs is None:
            fcs = {}
        self.buildz_unit = unit
        self.buildz_fcs = fcs
    def __setitem__(self, key, val):
        self.buildz_fcs[key] = val
    def __getitem__(self, key):
        return self.buildz_fcs[key]
    def __getattr__(self, key):
        if key in self.buildz_fcs:
            return self.buildz_fcs[key]
class WrapUnit(Unit):
    def add_conf(self, conf, tag=None):
        if self.mg is None:
            self.tmp_confs.append([conf, tag])
        else:
            super().add_conf(conf, tag)
    def bind(self, mg):
        if self.mg == mg:
            return
        super().bind(mg)
        for conf, tag in self.tmp_confs:
            self.add_conf(conf, tag)
        self.tmp_confs = []
        for fc in self.binds:
            fc(mg)
        self.binds = []
    def init(self, ns=None, deal_ns = None, env_ns = None, units=None):
        super().init(ns, deal_ns, env_ns)
        self.units = None
        self.tmp_confs = []
        self.fcs = {}
        self.binds = []
        self.wrap = WrapFcs(self, self.fcs)
        self.bind_units(units)
    def add_bind(self, fc):
        if self.mg is not None:
            fc(self.mg)
        else:
            self.binds.append(fc)
    def bind_units(self, units):
        if self.units == units:
            return
        self.units = units
        units.clone_fcs(self)
    def set_fc(self, key, fc):
        fc.bind(self)
        self.wrap[key] = fc

pass
class WrapUnits(Base):
    def clone_fcs(self, unit):
        for key, fc in self.fcs.items():
            unit.set_fc(key, fc.clone(unit))
    def bind(self, mg):
        self.mg = mg
        for key, unit in self.units.items():
            unit.bind(mg)
    def init(self):
        self.units = {}
        self.mg = None
        self.fcs = {}
    def set_fc(self, key, fc):
        self.fcs[key] = fc
    def unit(self, ns=None, deal_ns = None, env_ns = None):
        key = (ns, deal_ns, env_ns)
        if key not in self.units:
            unit = WrapUnit(ns, deal_ns, env_ns, self)
            if self.mg is not None:
                unit.bind(self.mg)
            self.units[key] = unit
        return self.units[key]
    def call(self, *a,**b):
        return self.unit(*a,**b)


from ..ioc.base import *
from ..ioc.confs import Confs
from ... import dz,pyz
class CacheLoads(Base):
    def init(self):
        self.loads = {}
    def call(self, src):
        if src not in self.loads:
            self.loads[src] = pyz.load(src)
        return self.loads[src]

pass
loads = CacheLoads()
class BaseEncape(Encape):
    load = loads
    def elist2obj(self, list, params=None):
        return [self.obj(k,params) for k in list]
    def edict2obj(self, list, params=None, dict=True):
        if dict:
            return {self.obj(k, params):self.obj(v, params) for k,v in list}
        else:
            return [[self.obj(it, params) for it in k] for k in list]
    @staticmethod
    def obj(val,*a,**b):
        if not isinstance(val, Encape):
            return val
        return val(*a,**b)

pass
class BaseDeal(Deal):
    load = loads
    def get_elist(self, unit, conf, key, default=[]):
        val = None
        if key in conf:
            val = conf[key]
        if val is None:
            val = default
        return self.list2encape(val, unit)
    def get_edict(self, unit, conf, key, default={}):
        val = None
        if key in conf:
            val = conf[key]
        if val is None:
            val = default
        return self.dict2encape(val, unit)
    def dict2encape(self, maps, unit):
        rst = []
        for k,v in dz.dict2iter(maps):
            rst.append((self.get_encape(k, unit),self.get_encape(v, unit)))
        return rst
    def list2encape(self, list, unit):
        list = [self.get_encape(k, unit) for k in list]
        return list
    def init(self):
        self.cache_encapes = {}
    def cache_get(self, key, ns):
        if key is None:
            return None
        key = (ns, key)
        return dz.get(self.cache_encapes, key, None)
    def cache_set(self, key, ns, encape):
        if key is None:
            return
        key = (ns, key)
        self.cache_encapes[key] = encape
    @staticmethod
    def get_encape(key, unit):
        if Confs.is_conf(key):
            ep,_,find = unit.get_encape(key, unit)
            assert find
            return ep
        return key
    def call(self, conf, unit):
        'encape, conf, conf_need_udpate'
        id,find=unit.conf_key(conf)
        ns = unit.ns
        encape = self.cache_get(id, ns)
        if encape is not None:
            return encape, conf, False
        conf, upd = self.update(conf, unit)
        encape = self.deal(conf,unit)
        self.cache_set(id, ns, encape)
        return encape,conf,upd

pass




from .. import base
from .. import itemz
import re
import json
class ValDeal(base.BaseDeal):
    def init(self, tp, fc):
        self._type  = tp
        self.fc = fc
    def deal(self, obj, conf):
        if not obj.check(is_val=1):
            return None
        val = obj.val
        if type(val)!=self._type:
            return None
        val = conf.s(self.fc(val))
        return itemz.ShowItem(val, obj.deep, is_val=1, is_done=1)

pass 



            

                



from .base import *
from ... import dz,pyz
from ..ioc.single import Single
class ValEncape(BaseEncape):
    '''
    '''
    def init(self, val):
        super().init()
        self.val = val
    def call(self, params=None, **maps):
        return self.val
class ValDeal(BaseDeal):
    def deal(self, conf, unit):
        assert 'val' in conf, f"[VAL] key 'val' not found in {conf}"
        return ValEncape(conf['val'])

pass


#

from buildz import iocz,xf, pyz, Base
profiles = xf.loads(r"""
xxx.test_var=profile_var_test
""")
wraps = iocz.build_wraps()
ns = wraps().wrap
@ns.obj(id='test_wrap')
@ns.obj.args("ref,xxx.test_var")
class Test(Base):
    def str(self):
        return f'Test(<{id(self)}>|id={self.id})'
    def init(self, id=0):
        super().init()
        self.id = id
    def call(self, val=None):
        if val is None:
            val = self
        print("Test.show:", val)
        return self

pass
ns.load_profiles(profiles)
#ns.obj.args("env,PATH")(Test)
#ns.obj(id='test')(Test)
var = 'test_var'
confs1 = r'''
ns: xxx
envs: {
    a=0
    b=1
}
confs: [
    [(val,test_var), conf_var_test]
]
confs.ns: [
    [[obj, test1], <buildz>.iocz.test.test.Test, null,{id=[cvar, <buildz>.iocz.test.test.var]}]
    {
        id=test
        type=obj
        source=<buildz>.iocz.test.test.Test
        single=1
        args=[
            #[ref, test1]
            #[env, PATH]
            [method, call,test_wrap]
        ]
        call: [method, call]
    }
    [[call,test_call], test, (0)]
]
'''.replace("<buildz>", "buildz")
def get_env_sys(self, id, sid=None):
    sysdt = os.getenv(id)
    return sysdt
def test():
    mg = iocz.build()
    wraps.bind(mg)
    print(mg)
    #unit = mg.add_conf(confs)
    unit = mg.add_conf(confs1)
    val,find=unit.get("test_call")
    print(f"test_call: {val,find}")
    exit()
    with mg.push_vars({"test": 123}):
        it, find = unit.get("test")
        print(f"it: {it, id(it)}, find: {find}")
    it, find = unit.get("test")
    print(f"it: {it, id(it)}, find: {find}")
    it, find = mg.get("test", "xxx")
    print(f"it: {it, id(it)}, find: {find}")
    print(f"env: {unit.get_env('b')}")
    print(type(it))
    it = Test(123)
    print(type(it))
    print(it)
    exit()

pyz.lc(locals(), test)



from ..ioc.base import *
from ... import dz, xf
def loads_xf(s):
    if type(s)!=str:
        return s
    return xf.loads(s)
class WrapItem(Base):
    def init(self, wrap, conf, add = False, remove=False):
        self.wrap = wrap
        self.add = add
        self.remove = remove
        self.conf = conf
    def call(self, cls):
        _id = id(cls)
        conf = self.wrap.update_wconf(_id, self.conf)
        if self.add:
            self.wrap.add_conf(conf)
            if self.remove:
                self.wrap.remove_wconf(_id)
        return cls
class WrapBase(Base):
    @staticmethod
    def loads(s):
        return loads_xf(s)
    @staticmethod
    def loads_list(arr):
        return [loads_xf(it) for it in arr]
    @staticmethod
    def loads_dict(maps):
        return {loads_xf(k):loads_xf(v) for k,v in maps.items()}
    def init(self, unit=None):
        self.wrap_conf = {}
        self.confs = []
        self.unit = None
        self.bind(unit)
    def clone(self, unit):
        return self.__class__(unit)
    def bind(self, unit):
        if self.unit == unit:
            return
        self.unit = unit
        for conf in self.confs:
            self.unit.add_conf(conf)
        self.confs = []
        return self
    def add_conf(self, conf):
        if self.unit is None:
            self.confs.append(conf)
        else:
            self.unit.add_conf(conf)
    def update_wconf(self, id, conf):
        if id not in self.wrap_conf:
            self.wrap_conf[id] = {}
        curr = self.wrap_conf[id]
        dz.fill(conf, curr)
        return curr
    def remove_wconf(self, id):
        if id in self.wrap_conf:
            del self.wrap_conf[id]



#
from buildz import Base
class MsgRecord(Base):
    """
        报文处理，非多线程安全
        要多线程安全，可以考虑：
            1，修改clone方法，新建一个MsgRecord实例而非返回self，并且自己加锁
            2，通过threading.current_thread().ident获取线程id，然后做相应处理
        需要实现的方法：
            _start: 新的请求/响应报文进来时调用，传入头部信息等
            _add: 传入报文体数据
            _add_chunk: 传入chunk数据
            _finish: 请求/响应报文数据已全部传入，看是否需要做什么处理
            实现可参考MsgLog
    """
    TypeReq ="request"
    TypeRsp = "response"
    def clone(self):
        return self
    def ssl_update(self):
        pass
    def init(self):
        self.ssl = False
        self.ssl_update()
    def set_ssl(self, ssl=True):
        '''
            声明当前连接转ssl或者从ssl转tcp，看是否需要在ssl_update做什么处理
            不管连接是ssl还是tcp，传入start，add，add_chunk方法里的数据都是明文
        '''
        self.ssl = ssl
        self.ssl_update()
    def start(self, msg_type, line, headers, data_size):
        self._start(msg_type, line, headers, data_size)
        return self
    def add(self, data):
        self._add(data)
        return self
    def add_chunk(self, data):
        self._add_chunk(data)
        return self
    def finish(self):
        self._finish()
        return self
    def request(self, line, headers, data_size):
        return self.start(MsgRecord.TypeReq, line, headers, data_size)
    def response(self, line, headers, data_size):
        return self.start(MsgRecord.TypeRsp, line, headers, data_size)
    def _start(self, msg_type, line, headers, data_size):
        """
        处理新报文数据
        line:
            msg_type==MsgRecord.TypeReq:
                line = http_type, url, protocol
            msg_type==MsgRecord.TypeRsp:
                line = protocol, code, rsp_text
            http_type: GET,POST,...
            protocol: 应该都是HTTP/1.1或HTTP/1.0
            code: 响应数据编码,200、404等
            rsp_text: 响应数据文本, OK, Not Found等
        headers: http头部信息, dict（字典）数据
        data_size: 当前报文的报文体数据大小（不包括chunk部分）
        """
        pass
    def _add(self, data):
        """
            报文体数据填充
        """
        pass
    def _add_chunk(self, data):
        """
            报文分块(chunked)传输，新增chunk数据
        """
        self._add(data)
    def _finish(self):
        """
            当前报文数据已经全部获取完成，要怎么处理可以执行了
            注：_add和_add_chunk返回的只是抓取到的报文体和块数据，如果报文体或块数据被编码和压缩了，需要自己处理解码和解压
        """
        pass

pass
class MsgLog(MsgRecord):
    def clone(self):
        return MsgLog(self.log)
    def init(self, log):
        self.log = log
        super().init()
        self.data = None
    def ssl_update(self):
        if self.ssl:
            self.log = self.log("https msg")
        else:
            self.log = self.log("http msg")
    def _start(self, msg_type, line, headers, data_size):
        self.data = [msg_type, line, headers, data_size, 0]
    def _add(self, data):
        self.data[-1]+=len(data)
    def _finish(self):
        msg_type, line, headers, data_size, real_size = self.data
        self.log.info(f"{msg_type} line:{line}, headers: {headers}, data_size: {data_size}/{real_size}")


#coding=utf-8
from ..ioc.base import Base, EncapeData, IdNotFoundError
from .base import FormatData,FormatDeal
from buildz import xf, pyz
import os
dp = os.path.dirname(__file__)
join = os.path.join
class RefDeal(FormatDeal):
    """
        引用ref:
            {
                id: id
                type: ref
                ref|key: 引导数据id
                default: 默认值
                info: item_conf, 额外的引用信息, 默认null
                force_new: 无视single配置创建新实例，默认false
            }
        简写:
            [[ref, id], [key, force_new], default, info]
            [[ref, id], key, default, info]
        极简:
            [ref, key]
        例:
            [ref, obj.test] // 数据项"obj.test"的引用
    """
    def init(self, fp_lists=None, fp_defaults=None):
        super().init("RefDeal", fp_lists, fp_defaults, join(dp, "conf", "ref_lists.js"), None)
    def deal(self, edata:EncapeData):
        data = edata.data
        data = self.fill(data)
        key = xf.get_first(data, 'ref', 'key')
        key = self.get_obj(key, edata.conf, src = edata.src)
        info = xf.g(data, info=None)
        if info is not None and type(info)==dict:
            #info = {k:self.get_obj(info, edata.conf, src = edata.src) for k in info}
            info = {'type':'map', 'data':info}
            info = self.get_obj(info, edata.conf, src = edata.src)
        else:
            info = {}
        _info = edata.info
        if type(_info)==dict:
            xf.deep_update(info, _info, 1)
        var, exist = edata.conf.get_var(key)
        if exist:
            return var
        try:
            return edata.conf.get_obj(key, info = info, src = edata.src)
        except IdNotFoundError as exp:
            #print(f"[testz] ref data: {data} in {edata.conf}")
            if "default" in data:
                key = data['default']
                return self.get_obj(key, edata.conf, src=edata.src)
            raise exp

pass




from . import buffer
from . import base
from . import pos
from . import exp
from ..stack import Stack
class Manager:
    def fcs(self, fc, *args, **maps):
        return fc(*self.cs(*args), **maps)
    def add_fc(self, fc, *args, **maps):
        obj = self.fcs(fc, *args, **maps)
        self.add(obj)
        return self
    def cs(self, *args):
        return [self.c(k) for k in args]
    def c(self, s):
        if self.bts:
            s = s.encode(self.code)
        return s
    # 上面这些没用
    def add(self,obj):
        #obj.init()
        obj.regist(self)
        if obj.has_prev():
            self.prevs.append(obj.prev)
        if obj.has_deal():
            self.deals.append(obj.deal)
        return self
    def do(self, fcs, *argv, **maps):
        for fc in fcs:
            if fc(*argv, **maps):
                return True
        return False
    def regist(self):
        id = self.index
        self.index+=1
        return "id_"+str(id)
    def __init__(self):
        self.index = 0
        self.deals = []
        self.prevs = []
    def load(self, reader):
        if type(reader) in [str, bytes]:
            #print(f"try str, {len(reader)}")
            buff = buffer.StrBuffer(reader)
            s = reader.replace(" ", "")
            queue = Stack(len(s)+1)
        else:
            buff = buffer.Buffer(reader)
            queue = []
        _pos = pos.PosCal()
        self.pos = _pos
        self.buffer = buff
        self.queue = queue
        import time
        crr = time.time()
        while self.do(self.prevs, buff, queue, _pos):
            pass
        now = time.time()
        #print(f"prev time: {now - crr}, prevs: {len(self.prevs)}, queue: {len(queue)}")
        #stack = []
        stack = Stack(len(queue))
        #stack = []
        crr = time.time()
        ns = 0
        while self.do(self.deals, queue, stack):
            ns = max(len(stack), ns)
            pass
        now = time.time()
        #print(f"deal time: {now - crr}, deals: {len(self.deals)}, stack: {ns}")
        if len(stack)==0:
            raise Exception("ERROR not data")
        for _item in stack:
            if not _item.check(is_val=1):
                print(f"err item: {_item}")
                raise exp.FormatExp("format error found", _item.pos)
        if len(stack)==1:
            return stack[0].val
        else:
            return [it.val for it in stack]

pass




from .base import *
from ... import dz,pyz
from ..ioc.single import Single
class DealEncape(BaseEncape):
    def init(self, src, targets, tag, prev_call, unit):
        super().init()
        self.src, self.targets, self.tag, self.prev_call, self.unit = src, targets, tag, prev_call, unit
    def call(self, params=None, **maps):
        #src = self.src
        src = self.obj(self.src)
        if self.prev_call:
            src = src()
        for target in self.targets:
            self.unit.set_deal(target, src, self.tag)
class DealDeal(BaseDeal):
    def deal(self, conf, unit):
        id,id_find = unit.conf_key(conf)
        src, tag, prev_call = dz.g(conf, source=None, tag=None, call=False)
        targets = dz.g(conf, deals=[])
        if type(targets) not in (list, tuple):
            targets = [targets]
        if Confs.is_conf(src):
            src = self.get_encape(src, unit)
        elif type(src)==str:
            src = self.load(src)
        return DealEncape(src, targets, tag, prev_call, unit,)

pass


from .. import base
from .. import item
from .. import exp
class LRDeal(base.BaseDeal):
    def check_right(self, obj):
        if obj.type!='str':
            return False
        val = obj.val
        if val.find(self.right)>=0:
            raise Exception(f"unexcept right symbol {self.right}")
        return True
    def labels(self):
        return [self.left]
    def types(self):
        return []
    def deal(self, buffer, rst, mg):
        cl = buffer.read(self.ll)
        if self.left!=cl:
            return False
        _arr = []
        rm = buffer.full().strip()
        buffer.clean2read(self.ll)
        if len(rm)>0:
            _arr.append(item.Item(rm, type="str", is_val=False))
        while True:
            cr = buffer.read(self.lr)
            if cr == self.right:
                rm = buffer.full().strip()
                buffer.clean2read(self.lr)
                break
            if not mg.deal(buffer, _arr):
                raise Exception("Error lr")
        buffer.clean()
        if len(rm)>0:
            _arr.append(item.Item(rm ,type = 'str', is_val=False))
        # dts = []
        # for k in _arr:
        #     _k = mg.build(k)
        #     if item.is_null(_k):
        #         continue
        #     dts.append(_k)
        #dts = [mg.build(k) for k in _arr]
        if self.mg_build:
            dts = mg.build_arr(_arr)
        else:
            dts = _arr
        obj = self.build_arr(dts)
        rst.append(obj)
        return True
    def to_vals(self, _arr, mg):
        dts = []
        for k in _arr:
            _k = mg.build(k)
            if item.is_null(_k):
                continue
            dts.append(_k)
        return dts
    def build_arr(self, obj):
        return item.Item(obj, type = self.type, is_val=True)
    """
        分隔符，有分隔符后将缓存的数据当作字符串
    """
    def prepare(self, mg):
        self.left = mg.like(self.left)
        self.right = mg.like(self.right)
        self.ll = len(self.left)
        self.lr = len(self.right)
    def init(self, left, right, name= "lr", mg_build = True):
        self.left = left
        self.right = right
        self.ll = len(left)
        self.lr = len(right)
        self.name = name
        self.mg_build = mg_build
    def err(self, s):
        return s.replace("<lr>", self.name)

pass



from .. import base
from .. import itemz
class ListDeal(base.BaseDeal):
    def height_tree(self, mg, arr):
        rst = []
        height = 1
        for obj in arr:
            _h = mg.height_tree(obj)
            height = max(height, _h[0]+1)
            rst.append(_h)
        return [height, rst]
    def init(self, left, right, spt):
        self.left = left
        self.right = right
        self.spt = spt
    def deal(self, mg, arr, up_height, heights, conf, deep, top_not_format):
        rst = []
        height = heights[0]
        for obj, _heights in zip(arr, heights[1]):
            sitem = mg.deal(obj, height, _heights, conf, deep+1)
            rst.append(sitem)
        return self.deal_encape(rst, conf, deep, height,up_height, self.left, self.right, top_not_format)

pass 



            

                


from ..iocz.conf import conf
from .. import Base, xf, dz,pyz
from .callz import *
from ..iocz.ioc_deal.base import BaseEncape,BaseDeal
from ..iocz.conf.up import BaseConf
from .conf_argz import *
from .evalx import *
from .. import iocz
from . import argz
class EnvFc(Fc):
    def init(self, unit):
        super().init()
        self.unit = unit
    def call(self, params):
        self.unit.update_env(params.maps)
        return True,1
class ArgsCallEncape(BaseEncape):
    def init(self, fcs, args, eval, unit):
        super().init()
        self.fcs = fcs
        self.args = args
        self.eval = eval
        self.call_fc = None
        self.unit = unit
    def ref(self, key, unit):
        key = self.obj(key)
        if type(key)==str:
            rst, find = unit.get(key)
            assert find, f"fc not found: '{key}'"
            key=rst
        return key
    def call(self, params=None):
        if self.call_fc is None:
            fcs = [self.ref(fc, self.unit) for fc in self.fcs]
            self.call_fc = Call(fcs, self.args, self.eval)
        return self.call_fc
class IOCZVarCallEncape(BaseEncape):
    def init(self, fc, args, eval, unit):
        if not dz.islist(fc):
            fc = [fc]
        self.fcs = fc
        self.args = args
        self.eval = eval
        self.unit = unit
    def call(self, params=None,**maps):
        if params is None:
            params = iocz.Params()
        inputs = params.inputs
        if inputs is None:
            inputs = argz.Params()
        if not cal_eval(self.eval, inputs):
            return None
        inputs = cal_args(self.args, inputs)
        params.inputs = inputs
        return deal_exp(self.deal, params, self.args)
    def deal(self, params):
        with self.unit.push_vars(params.inputs.maps):
            val = None
            for fc in self.fcs:
                if type(fc)==str:
                    fc, _tag, find = self.unit.get_encape(fc)
                    assert find, f"fc '{fc}' not found"
                _val = fc(params)
                val = pyz.nnull(_val, val)
            return val
class ArgsCallDeal(BaseDeal):
    def init(self, fc=True, update_env = False, fc_encape=None):
        super().init()
        _conf = BaseConf()
        index=1
        self.fc = fc
        self.update_env = update_env
        if fc_encape is None:
            fc_encape = ArgsCallEncape
        self.fc_encape = fc_encape
        if fc:
            _conf.ikey(1, 'fc', 'fcs,calls,call'.split(','))
            _conf.ikey(2, 'eval', "judges,judge,evals".split(","))
            _conf.ikey(3, 'conf')
        else:
            _conf.ikey(2, 'eval', "judges,judge,evals".split(","))
            _conf.ikey(1, 'conf')
        self.update=_conf
        self.args_builder = FullArgsBuilder()
    def deal(self, conf, unit):
        conf_eval = dz.g(conf, eval=None)
        conf_args = dz.g(conf, conf=None)
        _eval = None
        if conf_eval is not None:
            _eval = evalBuilder.default_build(UnitBuild(unit))(conf_eval)
        _args = None
        if conf_args is not None:
            _args = self.args_builder(conf_args)
        if self.fc:
            fcs = dz.g(conf, fc=[])
            if type(fcs)==str:
                fcs = [fcs]
            fcs = [self.get_encape(fc, unit) for fc in fcs]
        else:
            if self.update_env:
                fcs = [EnvFc(unit)]
            else:
                fcs = [RetFc()]
        return self.fc_encape(fcs, _args, _eval, unit)
class RetArgsDeal(ArgsCallDeal):
    def init(self):
        super().init(False)
class UpdateEnvDeal(ArgsCallDeal):
    def init(self):
        super().init(False,True)

class IOCZVarCallDeal(ArgsCallDeal):
    def init(self):
        super().init(fc_encape=IOCZVarCallEncape)
confs = xf.loads(r"""
# confs.pri: {
#     deal_args: {
#         type=deal
#         src=buildz.argzx.conf_callz.ArgsCallDeal
#         call=1
#         deals=[call,fc]
#     }
# }
confs.pri: [
    (
        (deal, deal_ret)
        buildz.argz.conf_callz.RetArgsDeal
        1,
        [argz_ret,ret]
    )
    (
        (deal, deal_argz_env)
        buildz.argz.conf_callz.UpdateEnvDeal
        1,
        [argz_env,argz_envs]
    )
    (
        (deal, deal_args)
        buildz.argz.conf_callz.ArgsCallDeal
        1,
        [argz,argz_call,argz_fc]
    )
    (
        (deal, deal_var)
        buildz.argz.conf_callz.IOCZVarCallDeal
        1,
        [vargz, argz_var]
    )
]
builds: [deal_args, deal_ret, deal_argz_env, deal_var]
""")
class ConfBuilder(Base):
    def init(self, conf=None):
        mg = iocz.build(conf)
        mg.add_conf(confs)
        self.mg = mg
        for key in 'push_var,push_vars,pop_var,pop_vars,set_var,set_vars,get_var,unset_var,unset_vars'.split(","):
            setattr(self, key, getattr(mg,key))
    def adds(self, conf):
        if type(conf)==str:
            conf = xf.loads(conf)
        conf = {'confs': conf}
        self.mg.add_conf(conf)
    def add(self, conf):
        if type(conf)==str:
            conf = xf.loads(conf)
        conf = {'confs': [conf]}
        self.mg.add_conf(conf)
    def call(self, key, args=[], maps={}):
        params = argz.Params(args, maps)
        #p = iocz.Params(params=params)
        fc, find = self.mg.get(key)
        #rst, find = self.mg.get(key, params=p)
        assert find, f"key not found: '{key}'"
        return fc(params)[0]
        return rst[0]
    def get(self, key, args=[], maps={}, ns=None):
        inputs = argz.Params(args, maps)
        params = iocz.Params(inputs=inputs)
        obj, find = self.mg.get(key, params=params)
        assert find, f"key not found: '{key}'"
        return obj



#
from ..ioc.base import Base, EncapeData

class DealDemo(Base):
    def init(self):
        pass
    def deal(self, edata:EncapeData):
        data = edata.data
        src = edata.src
        conf = edata.conf
        sid = edata.sid
        confs = edata.confs
        return {'type': edata.type, 'id': confs.get_data_id(data), 'conf': sid}

pass
